package com.a3l.sample.signin;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.amazon.A3L.authentication.A3LAuth;
import com.amazon.A3L.authentication.api.signin.A3LSignIn;
import com.amazon.A3L.authentication.api.signin.A3LSignInAccount;
import com.amazon.A3L.authentication.api.signin.A3LSignInClient;
import com.amazon.A3L.authentication.api.signin.A3LSignInOptions;
import com.google.android.gms.common.SignInButton;
import com.amazon.A3L.authentication.common.api.APIException;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;


public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    private static final String TAG = "SignInActivity";

    // Activity Request Code. 9001 is the default value for authentication.
    private static final int RC_SIGN_IN = 9001;

    private A3LSignInClient a3LSignInClient;
    private TextView mStatusTextView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Initialize A3LAuthentication with App Client ID and in case willing to use Custom Tab based
        // authentication in both Google And FireOS devices, set the second argument to true. If not,
        // it can be set false or just run init with a single argument.
        try {
            A3LAuth.init(getString(R.string.app_client_id), true);
        } catch (APIException e) {
            e.printStackTrace();
        }

        mStatusTextView = findViewById(R.id.status);

        findViewById(R.id.sign_out_button).setOnClickListener(this);
        findViewById(R.id.revoke_access).setOnClickListener(this);

        SignInButton signInButton = findViewById(R.id.sign_in_button);
        signInButton.setOnClickListener(this);
        signInButton.setStyle(1, 0);

        // Create A3LSignInOptions for setting up the properties for Authenticating a user.
        A3LSignInOptions a3LSignInOptions = new A3LSignInOptions.Builder(A3LSignInOptions.DEFAULT_SIGN_IN)
                .requestEmail()
                //.requestIdToken(getString(R.string.server_client_id)) // Optional (Add only if id token is necessary)
                .build();

        // Fetch a A3LSignInClient instance for the current application and set A3LSignInOptions
        a3LSignInClient = A3LSignIn.getClient(getApplicationContext(), a3LSignInOptions);
    }

    @Override
    public void onStart() {
        super.onStart();

        // Fetch A3LSignInAccount for the last signed in account to the app.
        A3LSignInAccount account =  A3LSignIn.getLastSignedInAccount(this);
        if(account == null) {
            updateUI(null);
        } else {
            updateUI(account);
            return;
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == RC_SIGN_IN) {

            // Invocation of Network call to Google Auth Server by passing the Access Token and
            // fetching profile data
            Task<A3LSignInAccount> task = A3LSignIn.getSignedInAccountFromIntent(data);
            handleSignInResult(task);
        }
    }

    private void handleSignInResult(Task<A3LSignInAccount> completedTask) {
        // Add an OnCompleteListener to the task to fetch the account from the task post execution.
        completedTask.addOnCompleteListener(new OnCompleteListener<A3LSignInAccount>() {
            @Override
            public void onComplete(@NonNull Task<A3LSignInAccount> task) {
                try {
                    A3LSignInAccount account = task.getResult();
                    updateUI(account);
                } catch (Exception e) {
                    Log.w(TAG, "signInResult:failed code=" + e.getMessage());
                    e.printStackTrace();
                    updateUI(null);
                }
            }
        });
    }

    private void signIn() throws APIException {
        // Fetch a signInIntent for the current A3LSignInClient
        Intent signInIntent = a3LSignInClient.getSignInIntent();
        startActivityForResult(signInIntent, RC_SIGN_IN);
    }

    private void signOut() {
        // Signout of the current app
        Task<Void> task = a3LSignInClient.signOut();
        task.addOnCompleteListener(new OnCompleteListener<Void>() {
            @Override
            public void onComplete(@NonNull Task<Void> task) {
                updateUI(null);
            }
        });
    }

    private void revokeAccess() {
        // Invocation of Network call to Google Auth Server by passing the Refresh Token to
        // revoking access of the user from the app
        Task<Void> task = a3LSignInClient.revokeAccess();
        task.addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void unused) {
                updateUI(null);
            }
        });
        task.addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void updateUI(@Nullable A3LSignInAccount account) {
        if (account != null) {
            mStatusTextView.setText(getString(R.string.signed_in_fmt, account.getDisplayName()));
            findViewById(R.id.email).setVisibility(View.VISIBLE);
            findViewById(R.id.firstName).setVisibility(View.VISIBLE);
            findViewById(R.id.lastName).setVisibility(View.VISIBLE);
            TextView emailView = findViewById(R.id.email);
            TextView firstNameView = findViewById(R.id.firstName);
            TextView lastNameView = findViewById(R.id.lastName);
            emailView.setText(String.format("Email Id: %s", account.getEmail()));
            firstNameView.setText(String.format("Given Name: %s", account.getGivenName()));
            lastNameView.setText(String.format("Family Name: %s", account.getFamilyName()));
            findViewById(R.id.sign_in_button).setVisibility(View.GONE);
            findViewById(R.id.sign_out).setVisibility(View.VISIBLE);
        } else {
            mStatusTextView.setText(R.string.signed_out);
            findViewById(R.id.sign_in_button).setVisibility(View.VISIBLE);
            findViewById(R.id.sign_out).setVisibility(View.GONE);
            findViewById(R.id.email).setVisibility(View.GONE);
            findViewById(R.id.firstName).setVisibility(View.GONE);
            findViewById(R.id.lastName).setVisibility(View.GONE);
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.sign_in_button:
                try {
                    signIn();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                break;
            case R.id.sign_out_button:
                signOut();
                break;
            case R.id.revoke_access:
                revokeAccess();
                break;
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }
}