package com.a3l.sample.signin

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.TextView
import com.amazon.A3L.authentication.A3LAuth
import com.amazon.A3L.authentication.api.signin.A3LSignIn
import com.amazon.A3L.authentication.api.signin.A3LSignInAccount
import com.amazon.A3L.authentication.api.signin.A3LSignInClient
import com.amazon.A3L.authentication.api.signin.A3LSignInOptions
import com.amazon.A3L.authentication.common.api.APIException
import com.google.android.gms.common.SignInButton
import com.google.android.gms.tasks.Task
import java.lang.Exception

class MainActivity : AppCompatActivity(), View.OnClickListener {
    private var a3LSignInClient: A3LSignInClient? = null
    private var mStatusTextView: TextView? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        // Initialize A3LAuthentication with App Client ID and in case willing to use Custom Tab based
        // authentication in both Google And FireOS devices, set the second argument to true. If not,
        // it can be set false or just run init with a single argument.
        try {
            A3LAuth.init(getString(R.string.app_client_id), false)
        } catch (e: APIException) {
            e.printStackTrace()
        }
        mStatusTextView = findViewById(R.id.status)
        findViewById<View>(R.id.sign_out_button).setOnClickListener(this)
        findViewById<View>(R.id.revoke_access).setOnClickListener(this)
        val signInButton = findViewById<SignInButton>(R.id.sign_in_button)
        signInButton.setOnClickListener(this)
        signInButton.setStyle(1, 0)


        // Create A3LSignInOptions for setting up the properties for Authenticating a user.
        val a3LSignInOptions = A3LSignInOptions.Builder(A3LSignInOptions.DEFAULT_SIGN_IN)
            .requestEmail()
            .requestIdToken(getString(R.string.server_client_id)) // Optional (Add only if id token is necessary)
            .build()

        // Fetch a A3LSignInClient instance for the current application and set A3LSignInOptions
        a3LSignInClient = A3LSignIn.getClient(applicationContext, a3LSignInOptions)
    }

    public override fun onStart() {
        super.onStart()

        // Fetch A3LSignInAccount for the last signed in account to the app.
        val account = A3LSignIn.getLastSignedInAccount(this)
        if (account == null) {
            updateUI(null)
        } else {
            updateUI(account)
            return
        }
    }

    public override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == RC_SIGN_IN) {

            // Invocation of Network call to Google Auth Server by passing the Access Token and
            // fetching profile data
            val task = A3LSignIn.getSignedInAccountFromIntent(data!!)
            handleSignInResult(task)
        }
    }

    private fun handleSignInResult(completedTask: Task<A3LSignInAccount>) {
        // Add an OnCompleteListener to the task to fetch the account from the task post execution.
        completedTask.addOnCompleteListener { task ->
            try {
                val account = task.result
                updateUI(account)
            } catch (e: Exception) {
                Log.w(TAG, "signInResult:failed code=" + e.message)
                e.printStackTrace()
                updateUI(null)
            }
        }
    }

    @Throws(APIException::class)
    private fun signIn() {
        // Fetch a signInIntent for the current A3LSignInClient
        val signInIntent = a3LSignInClient!!.signInIntent
        startActivityForResult(signInIntent, RC_SIGN_IN)
    }

    private fun signOut() {
        // Signout of the current app
        val task = a3LSignInClient!!.signOut()
        task.addOnCompleteListener { updateUI(null) }
    }

    private fun revokeAccess() {
        // Invocation of Network call to Google Auth Server by passing the Refresh Token to
        // revoking access of the user from the app
        val task = a3LSignInClient!!.revokeAccess()
        task.addOnSuccessListener { updateUI(null) }
        task.addOnFailureListener { e -> e.printStackTrace() }
    }

    private fun updateUI(account: A3LSignInAccount?) {
        if (account != null) {
            mStatusTextView!!.text = getString(R.string.signed_in_fmt, account.displayName)
            findViewById<View>(R.id.email).visibility = View.VISIBLE
            findViewById<View>(R.id.firstName).visibility = View.VISIBLE
            findViewById<View>(R.id.lastName).visibility = View.VISIBLE
            val emailView = findViewById<TextView>(R.id.email)
            val firstNameView = findViewById<TextView>(R.id.firstName)
            val lastNameView = findViewById<TextView>(R.id.lastName)
            emailView.text = String.format("Email Id: %s", account.email)
            firstNameView.text = String.format("Given Name: %s", account.givenName)
            lastNameView.text = String.format("Family Name: %s", account.familyName)
            findViewById<View>(R.id.sign_in_button).visibility = View.GONE
            findViewById<View>(R.id.sign_out).visibility = View.VISIBLE
        } else {
            mStatusTextView!!.setText(R.string.signed_out)
            findViewById<View>(R.id.sign_in_button).visibility = View.VISIBLE
            findViewById<View>(R.id.sign_out).visibility = View.GONE
            findViewById<View>(R.id.email).visibility = View.GONE
            findViewById<View>(R.id.firstName).visibility = View.GONE
            findViewById<View>(R.id.lastName).visibility = View.GONE
        }
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.sign_in_button -> try {
                signIn()
            } catch (e: Exception) {
                e.printStackTrace()
            }
            R.id.sign_out_button -> signOut()
            R.id.revoke_access -> revokeAccess()
        }
    }

    public override fun onDestroy() {
        super.onDestroy()
    }

    companion object {
        private const val TAG = "SignInActivity"

        // Activity Request Code. 9001 is the default value used for authentication.
        private const val RC_SIGN_IN = 9001
    }
}